@@ -10,6 +10,7 @@ import requests
 from flask import request, jsonify, current_app
 from redis.exceptions import RedisError
 from torch.utils.data import DataLoader
+from datetime import datetime
 
 from .dataset import _KubeArgs, KubeDataset
 from .exceptions import *
@@ -108,21 +109,10 @@ class KubeModel(ABC):
         self.logger.debug(f"Configure optimizer at epoch: {self.epoch}")
 
 
-    # def _load_optimizer_state(self):
-    #     """
-    #     Checks for a previously saved state of the optimizer and loads it
-    #     """
-    #     if os.path.isfile('opt.pkl'):
-    #         self.logger.debug('Loading optimizer state')
-    #         with open('opt.pkl', 'rb') as f:
-    #             self.logger.debug('Loading optimizer state, epoch: {self.epoch}')
-    #             state = pickle.load(f)
-    #             self.optimizer.load_state_dict(state)
-
 
     def _load_optimizer_state(self):
         """
-        Saves the optimizer state to be loaded again in
+        Saves the optimizer state to host machine path /output and it will be loaded again in
         the following epochs
         """
         job_id = self.args._job_id
@@ -145,22 +135,10 @@ class KubeModel(ABC):
         if self.optimizer is not None:
             self.optimizer.state = defaultdict(dict)
 
-    # def _save_optimizer_state(self):
-    #     """
-    #     Saves the optimizer state to be loaded again in
-    #     the following epochs
-    #     """
-        
-    #     self.logger.debug('saving optimizer state')
-
-    #     with open('opt.pkl', 'wb') as f:
-    #         self.logger.debug('saving optimizer state, epoch: {self.epoch}')
-    #         pickle.dump(self.optimizer.state_dict(), f)
-    #     print('saved state')
 
     def _save_optimizer_state(self):
         """
-        Saves the optimizer state to be loaded again in
+        Saves the optimizer state to host machine path /output and it will be loaded again in
         the following epochs
         """
         job_id = self.args._job_id
@@ -175,7 +153,7 @@ class KubeModel(ABC):
 
     def _save_file_test(self):
         """
-        Saves the optimizer state to be loaded again in
+        Saves the test file to be loaded again in
         the following epochs
         """
         job_id = self.args._job_id
@@ -190,7 +168,7 @@ class KubeModel(ABC):
 
     def _load_file_test(self):
         """
-        Saves the optimizer state to be loaded again in
+        Saves the file state to be loaded again in
         the following epochs
         """
         job_id = self.args._job_id
@@ -215,7 +193,7 @@ class KubeModel(ABC):
         weight_key = f'{job_id}:optimizer:{func_id}' 
         pickled_val  = pickle.dumps(self.optimizer.state_dict())
 
-        self.logger.debug(f"Saving optimizer value: {pickled_val}")
+        #self.logger.debug(f"Saving optimizer value: {pickled_val}")
 
         self._redis_client.set(weight_key, pickled_val)
 
@@ -233,10 +211,11 @@ class KubeModel(ABC):
         weight_key = f'{job_id}:optimizer:{func_id}' 
 
         if  self._redis_client.exists(weight_key):
-            epoch = self.args.epoch
-            weight_key_print = f'{job_id}:print:{func_id}' 
-            value = f'{job_id}:print:{func_id}/{epoch}' 
-            self._redis_client.set(weight_key_print, value)
+            # used for testing redis
+            # epoch = self.args.epoch
+            # weight_key_print = f'{job_id}:print:{func_id}' 
+            # value = f'{job_id}:print:{func_id}/{epoch}' 
+            # self._redis_client.set(weight_key_print, value)
 
             pickled_val = self._redis_client.get(weight_key)
             opt_states = pickle.loads(pickled_val)  
@@ -354,11 +333,16 @@ class KubeModel(ABC):
         reset optimizer state
         :return:
         """
+        startModelLoading = datetime.now()
         self.__load_model()
-        # self._load_optimizer_redis()
+        self.logger.debug(f"Model loading Time, {datetime.now() - startModelLoading}")
+        startOptimizerLoading = datetime.now()
+        self._load_optimizer_redis()
+        self.logger.debug(f"Optimizer loading Time, {datetime.now() - startOptimizerLoading}")
+
         # self._load_redis_test()
         # self._load_file_test()
-        self._load_optimizer_state()
+        # self._load_optimizer_state()
 
         #self._load_optimizer_state()
 
@@ -369,11 +353,15 @@ class KubeModel(ABC):
         Called at the end of each iteration
         :return:
         """
+        startModelSaving = datetime.now()
         self.__save_model()
-        # self._save_optimizer_redis()
+        self.logger.debug(f"Model saving Time, {datetime.now() - startModelSaving}")
+        startoptimizerSaving = datetime.now()
+        self._save_optimizer_redis()
+        self.logger.debug(f"Optimizer saving Time, {datetime.now() - startoptimizerSaving}")
         # self._save_redis_test()
         # self._save_file_test()
-        self._save_optimizer_state()
+        # self._save_optimizer_state()
 
     def _batch_to_device(self, batch: Union[torch.Tensor, Iterable[torch.Tensor]]):
         """
@@ -408,27 +396,36 @@ class KubeModel(ABC):
 
         :return: The loss of the epoch, as returned by the user function
         """
-        self.logger.debug(f"----------------------Epoch: {self.epoch} Starts---------------------------\n")
+        self.logger.debug(f"----------------------Epoch: {self.epoch} Starts---------------------------")
         self._on_train_start()
 
+
+
         # Determine the batches that we need to train on and the first subset id
         assigned_subsets = split_minibatches(range(self._dataset.num_docs),
                                              self.args._N)[self.args._func_id]
 
         # calculate the number of subsets that we need to train on
         # per epoch
+
+        startDataLPrepartion = datetime.now()
+
         subsets_per_iter = get_subset_period(self.args._K,
                                              self.args.batch_size,
                                              assigned_subsets)
         self.logger.debug(f"Subsets per iteration: {subsets_per_iter}")
         intervals = range(assigned_subsets.start, assigned_subsets.stop, subsets_per_iter)
 
+        self.logger.debug(f"Data prepartion Time, {datetime.now() - startDataLPrepartion}")
+
         # the loss will be added cross intervals, each interval will have one loader, whose length
         # will determine the number of losses added.
         loss = 0
         num_iterations = 0
         for i in intervals:
 
+            startDataLoading = datetime.now()
+
             self.logger.debug(f"Starting iteration {i}")
             self._dataset._load_train_data(start=i, end=min(assigned_subsets.stop, i + subsets_per_iter))
 
@@ -436,16 +433,22 @@ class KubeModel(ABC):
             loader = DataLoader(self._dataset, batch_size=self.batch_size)
             num_iterations += len(loader)
 
+            self.logger.debug(f"Data Loading Time, {datetime.now() - startDataLoading}")
+
             # load the reference model, train and save
             try:
                 self._on_iteration_start()
 
+                startTraining = datetime.now()
+
                 for idx, batch in enumerate(loader):
                     # send the batch to the appropriate device
                     batch = self._batch_to_device(batch)
                     loss += self.train(batch, idx)
                     # self.logger.debug(f'loss is {loss}, iterations are {num_iterations}')
 
+                self.logger.debug(f"Training Time, {datetime.now() - startTraining}")
+
                 self._on_iteration_end()
             except RedisError as re:
                 raise StorageError(re)
